# See the bottom of the file for description, copyright and license information
package Foswiki::Plugins::MakeMeAMemberPlugin;

use strict;
require Foswiki::Func;

our $VERSION = '$Rev: 0 (1970-01-01) $';
our $RELEASE = '1.0.0';
our $SHORTDESCRIPTION =
'Allows selected users to promote themselves to aMember status(and demote themselves again).';
our $NO_PREFS_IN_TOPIC = 1;

sub initPlugin {
    Foswiki::Func::registerTagHandler( 'MAKE_ME_A_MEMBER', \&_DEIFY );

    return 1;
}

sub _DEIFY {
    my ( $session, $params, $topic, $web ) = @_;
    my $user = Foswiki::Func::getWikiName();
    my $group = $params->{_DEFAULT};

    my $status = 'makemeamember_out';
    # Abort if they can't change SuperAMemberGroup
    return '' unless (Foswiki::Func::checkAccessPermission(
			  'CHANGE', $user, undef,
			  $group, $Foswiki::cfg{UsersWebName}));

    my ($meta, $text) = Foswiki::Func::readTopic(
	$Foswiki::cfg{UsersWebName}, $group);
    my $pref = $meta->get('PREFERENCE', 'GROUP');
    my %members = map { $_ => 1 } split(/,\s*/, $pref->{value});
    my $iAmIn = $members{$user};

    my $query = Foswiki::Func::getCgiQuery();

    if ( $query->param('toggle_me') ) {
	$query->delete('toggle_me');
	if ($iAmIn) {
	    delete $members{$user};
	    $iAmIn = 0;
	} else {
	    $members{$user} = $iAmIn = 1;
	}
	$pref->{value} = join(',', keys %members);
	Foswiki::Func::saveTopic(
	    $Foswiki::cfg{UsersWebName},
	    $Foswiki::cfg{SuperAMemberGroup}, $meta, $text,
	    { forcenewrevision => 1,
	      minor => 1,
	      dontlog => 1 });
    }
    my $url = Foswiki::Func::getScriptUrl(
	$web, $topic, 'view',
	'toggle_me' => 1 );

    $status = 'makemeamember_in' if $iAmIn;
    Foswiki::Func::loadTemplate('makemeamember');
    my $template = Foswiki::Func::expandTemplate($status);
    $template =~ s/%URL%/$url/gs;
    return $template;
}

1;
__END__

Plugin for Foswiki - The Free and Open Source Wiki, http://foswiki.org/

Copyright (C) 2012 Crawford Currie http://c-dot.co.uk
and Foswiki Contributors. All Rights Reserved. Foswiki Contributors
are listed in the AUTHORS file in the root of this distribution.
NOTE: Please extend that file, not this notice.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version. For
more details read LICENSE in the root of this distribution.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

For licensing info read LICENSE file in the Foswiki root.

Author: Crawford Currie http://c-dot.co.uk
